@@ -1,10 +1,25 @@
-
 //! English/ASCII letter sets provided as u8, str arrays, and hashsets.
 
 use super::*;
 
+pub enum AsciiCharSet {
+    Letters,
+    LettersUppercase,
+    LettersLowercase,
+}
+
+impl AsciiCharSet {
+    pub const fn value(&self) -> &[u8] {
+        match *self {
+            AsciiCharSet::Letters => &ASCII_LETTERS,
+            AsciiCharSet::LettersUppercase => &ASCII_LETTERS_UPPERCASE,
+            AsciiCharSet::LettersLowercase => &ASCII_LETTERS_LOWERCASE,
+        }
+    }
+}
+
 // Upper and lower case
-pub const ASCII_LETTERS_U8: [u8; 52] = [
+pub const ASCII_LETTERS: [u8; 52] = [
     b'A', b'B', b'C', b'D', b'E', b'F', b'G', b'H', b'I', b'J', b'K', b'L', b'M', b'N', b'O', b'P',
     b'Q', b'R', b'S', b'T', b'U', b'V', b'W', b'X', b'Y', b'Z', b'a', b'b', b'c', b'd', b'e', b'f',
     b'g', b'h', b'i', b'j', b'k', b'l', b'm', b'n', b'o', b'p', b'q', b'r', b's', b't', b'u', b'v',
@@ -15,8 +30,11 @@ pub const ASCII_LETTERS_STR: [&str; 52] = [
     "T", "U", "V", "W", "X", "Y", "Z", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l",
     "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z",
 ];
+
+// pub const ASCII_LETTERS_RANGE = Range {ASCII_LETTERS};
+
 lazy_static! {
-    pub static ref ASCII_LETTERS_HASHSET_U8: HashSet<u8> = new_u8_hashset(&ASCII_LETTERS_U8);
+    pub static ref ASCII_LETTERS_HASHSET: HashSet<u8> = new_hashset(&ASCII_LETTERS);
 }
 lazy_static! {
     pub static ref ASCII_LETTERS_HASHSET_STR: HashSet<&'static str> =
@@ -24,7 +42,7 @@ lazy_static! {
 }
 
 // Upper case
-pub const ASCII_LETTERS_UPPERCASE_U8: [u8; 26] = [
+pub const ASCII_LETTERS_UPPERCASE: [u8; 26] = [
     b'A', b'B', b'C', b'D', b'E', b'F', b'G', b'H', b'I', b'J', b'K', b'L', b'M', b'N', b'O', b'P',
     b'Q', b'R', b'S', b'T', b'U', b'V', b'W', b'X', b'Y', b'Z',
 ];
@@ -33,8 +51,8 @@ pub const ASCII_LETTERS_UPPERCASE_STR: [&str; 26] = [
     "T", "U", "V", "W", "X", "Y", "Z",
 ];
 lazy_static! {
-    pub static ref ASCII_LETTERS_UPPERCASE_HASHSET_U8: HashSet<u8> =
-        new_u8_hashset(&ASCII_LETTERS_UPPERCASE_U8);
+    pub static ref ASCII_LETTERS_UPPERCASE_HASHSET: HashSet<u8> =
+        new_hashset(&ASCII_LETTERS_UPPERCASE);
 }
 lazy_static! {
     pub static ref ASCII_LETTERS_UPPERCASE_HASHSET_STR: HashSet<&'static str> =
@@ -42,7 +60,7 @@ lazy_static! {
 }
 
 // Lower case
-pub const ASCII_LETTERS_LOWERCASE_U8: [u8; 26] = [
+pub const ASCII_LETTERS_LOWERCASE: [u8; 26] = [
     b'a', b'b', b'c', b'd', b'e', b'f', b'g', b'h', b'i', b'j', b'k', b'l', b'm', b'n', b'o', b'p',
     b'q', b'r', b's', b't', b'u', b'v', b'w', b'x', b'y', b'z',
 ];
@@ -51,8 +69,8 @@ pub const ASCII_LETTERS_LOWERCASE_STR: [&str; 26] = [
     "t", "u", "v", "w", "x", "y", "z",
 ];
 lazy_static! {
-    pub static ref ASCII_LETTERS_LOWERCASE_HASHSET_U8: HashSet<u8> =
-        new_u8_hashset(&ASCII_LETTERS_LOWERCASE_U8);
+    pub static ref ASCII_LETTERS_LOWERCASE_HASHSET: HashSet<u8> =
+        new_hashset(&ASCII_LETTERS_LOWERCASE);
 }
 lazy_static! {
     pub static ref ASCII_LETTERS_LOWERCASE_HASHSET_STR: HashSet<&'static str> =
