@@ -1,28 +1,86 @@
-
 //! Quality character (sub-)sets including Phred33, Phred64, and Solexa/Illumina 1.0 (for compatibility). Provided as u8 and str arrays.
 
-use std::collections::HashMap;
 use super::*;
+use std::ops::RangeInclusive;
+
+/// Filler for non-coding values in encoding decoding, chosen because the value is not part of the set
+pub const PHRED_FILLER: u8 = 255;
+
+pub enum QualityCharSet {
+    Phred33,
+    Phred64,
+    Sanger,
+    Solexa,
+    Phred33Score,
+    Phred64Score,
+    SangerScore,
+}
+
+impl QualityCharSet {
+    pub const fn value(&self) -> &[u8] {
+        match *self {
+            QualityCharSet::Phred33 => &PHRED33_ENCODE,
+            QualityCharSet::Phred64 => &PHRED64_ENCODE,
+            QualityCharSet::Solexa => &SOLEXA_ENCODE,
+            QualityCharSet::Sanger => &SOLEXA_ENCODE,
+            QualityCharSet::Phred33Score => &PHRED33_SCORE,
+            QualityCharSet::Phred64Score => &PHRED64_SCORE,
+            QualityCharSet::SangerScore => &SANGER_SCORE,
+        }
+    }
+}
 
 /// Phred33 charset: ASCII 33-75
-pub const PHRED33_U8: [u8; 43] = [
+pub const PHRED33_ENCODE: [u8; 43] = [
     b'!', b'"', b'#', b'$', b'%', b'&', 0x0027, b'(', b')', b'*', b'+', b',', b'-', b'.', b'/',
     b'0', b'1', b'2', b'3', b'4', b'5', b'6', b'7', b'8', b'9', b':', b';', b'<', b'=', b'>', b'?',
     b'@', b'A', b'B', b'C', b'D', b'E', b'F', b'G', b'H', b'I', b'J', b'K'];
 
 /// Phred33 scores: 0-42
-pub const PHRED33_SCORES_U8: [u8; 43] = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42];
+pub const PHRED33_SCORE: [u8; 43] = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42];
 
-/// Phred33 charset: ASCII 33-73
+/// Phred33 DECODE scores. Array that has filler up til 33, where it starts at 0 for phred33 scores. Should not be used directly, as filler will give wrong information. We use it as "Check the ENCODE is valid phred33 (33-75), and if so get the DECODE from this array"
+pub const PHRED33_DECODE: [u8; 75] = [
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,
+    0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
+    16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,
+    31,32,33,34,35,36,37,38,39,40,41,42,
+];
+
+/// Phred33 charset: ASCII 33-75
     pub const PHRED33_STR: [&str; 43] = [
     r#"!"#, r#"""#, r##"#"##, r#"$"#, r#"%"#, r#"&"#, r#"'"#, r#"("#, r#")"#, r#"*"#, r#"+"#,
     r#","#, r#"-"#, r#"."#, r#"/"#, r#"0"#, r#"1"#, r#"2"#, r#"3"#, r#"4"#, r#"5"#, r#"6"#, r#"7"#,
     r#"8"#, r#"9"#, r#":"#, r#";"#, r#"<"#, r#"="#, r#">"#, r#"?"#, r#"@"#, r#"A"#, r#"B"#, r#"C"#,
     r#"D"#, r#"E"#, r#"F"#, r#"G"#, r#"H"#, r#"I"#, r#"J"#, r#"K"#];
 
+pub const PHRED33_MIN_USIZE: usize = 33;
+pub const PHRED33_MAX_USIZE: usize = 75;
+pub const PHRED33_SCORE_MIN_USIZE: usize = 0;
+pub const PHRED33_SCORE_MAX_USIZE: usize = 42;
+pub const PHRED33_MIN_U8: u8 = 33;
+pub const PHRED33_MAX_U8: u8 = 75;
+pub const PHRED33_SCORE_MIN_U8: u8 = 0;
+pub const PHRED33_SCORE_MAX_U8: u8 = 42;
+pub const PHRED33_MIN_U64: u64 = 33;
+pub const PHRED33_MAX_U64: u64 = 75;
+pub const PHRED33_SCORE_MIN_U64: u64 = 0;
+pub const PHRED33_SCORE_MAX_U64: u64 = 42;
+
+/// Phred33 range: 33-75
+pub const PHRED33_RANGE: RangeInclusive<usize> = PHRED33_MIN_USIZE..=PHRED33_MAX_USIZE;
+/// Phred33 score range: 0-42
+pub const PHRED33_SCORE_RANGE: RangeInclusive<usize> = PHRED33_SCORE_MIN_USIZE..=PHRED33_SCORE_MAX_USIZE;
+
 lazy_static! {
     /// Phred33 charset as hashset: ASCII 33-73
-    pub static ref PHRED33_HASHSET_U8: HashSet<u8> = new_u8_hashset(&PHRED33_U8);
+    pub static ref PHRED33_HASHSET: HashSet<u8> = new_hashset(&PHRED33_ENCODE);
 }
 lazy_static! {
     /// Phred33 charset as hashset: ASCII 33-73
@@ -31,7 +89,7 @@ lazy_static! {
 
 lazy_static!{
     /// This is the quality score shifted 33 so if the u8 is 33, the score is 0. We can look that up with this hashmap.
-    pub static ref PHRED33_HASHMAP_U8: HashMap<u8, u8> = vec![
+    pub static ref PHRED33_HASHMAP_DECODE: HashMap<u8, u8> = vec![
         (b'!', 0), (b'"', 1), (b'#', 2), (b'$', 3), (b'%', 4), (b'&', 5), (0x0027, 6), (b'(', 7), (b')', 8), (b'*', 9), (b'+', 10), (b',', 11), (b'-', 12), (b'.', 13), (b'/', 14),
         (b'0', 15), (b'1', 16), (b'2', 17), (b'3',18), (b'4', 19), (b'5', 20), (b'6', 21), (b'7', 22), (b'8', 23), (b'9', 24), (b':', 25), (b';', 26), (b'<', 27), (b'=', 28), (b'>', 29), (b'?', 30),
         (b'@', 31), (b'A', 32), (b'B', 33), (b'C', 34), (b'D', 35), (b'E', 36), (b'F', 37), (b'G', 38), (b'H', 39), (b'I', 40), (b'J', 41), (b'K', 42)
@@ -40,20 +98,15 @@ lazy_static!{
 
 lazy_static!{
     /// This is the quality score shifted 33 so if the u8 is 33, the score is 0. We can look that up with this hashmap.
-    pub static ref PHRED33_HASHMAP_ENCODE_U8: HashMap<u8, u8> = vec![
+    pub static ref PHRED33_HASHMAP_ENCODE: HashMap<u8, u8> = vec![
         (0 , b'!'), (1, b'"'), (2, b'#'), (3, b'$'), (4, b'%'), (5, b'&'), (6, 0x0027), (7, b'('), (8, b')'), (9, b'*'), (10, b'+'), (11, b','), (12, b'-'), (13, b'.'), (14, b'/'),
         (15, b'0'), (16, b'1'), (17, b'2'), (18, b'3'), (19, b'4'), (20, b'5'), (21, b'6'), (22, b'7'), (23, b'8'), (24, b'9'), (25, b':'), (26, b';'), (27, b'<'), (28, b'='), (29, b'>'), (30, b'?'),
         (31, b'@'), (32, b'A'), (33, b'B'), (34, b'C'), (35, b'D'), (36, b'E'), (37, b'F'), (38, b'G'), (39, b'H'), (40, b'I'), (41, b'J'), (42, b'K')
     ].into_iter().collect();
 }
 
-// lazy_static!{
-//     /// PHRED33 (key) to probability (value) hashmap.
-//     pub static ref PHRED33_PROBABILITY_HASHMAP_U8: HashMap<u8, u8> = vec![].into_iter().collect();
-// }
-
 /// Phred64 charset: ASCII 64-126
-pub const PHRED64_U8: [u8; 63] = [
+pub const PHRED64_ENCODE: [u8; 63] = [
     b'@', b'A', b'B', b'C', b'D', b'E', b'F', b'G', b'H', b'I', b'J', b'K', b'L', b'M', b'N', b'O',
     b'P', b'Q', b'R', b'S', b'T', b'U', b'V', b'W', b'X', b'Y', b'Z', 0x005B, 0x005C, 0x005D, b'^',
     b'_', b'`', b'a', b'b', b'c', b'd', b'e', b'f', b'g', b'h', b'i', b'j', b'k', b'l', b'm', b'n',
@@ -61,8 +114,29 @@ pub const PHRED64_U8: [u8; 63] = [
 ];
 
 /// Phred64 scores: 0-62
-pub const PHRED64_SCORES_U8: [u8; 63] = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62];
+pub const PHRED64_SCORE: [u8; 63] = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62];
 
+/// Phred64 DECODE scores. Array that has filler up til 64, where it starts at 0 for phred64 scores. Should not be used directly, as filler will give wrong information. We use it as "Check the ENCODE is valid phred64 (64-126), and if so get the DECODE from this array"
+pub const PHRED64_DECODE: [u8; 127] = [
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,PHRED_FILLER,
+    0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
+    16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,
+    31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,
+    46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,
+    61,62
+];
 
 /// Phred64 charset: ASCII 64-126
 pub const PHRED64_STR: [&str; 63] = [
@@ -73,9 +147,27 @@ pub const PHRED64_STR: [&str; 63] = [
     r#"p"#, r#"q"#, r#"r"#, r#"s"#, r#"t"#, r#"u"#, r#"v"#, r#"w"#, r#"x"#, r#"y"#, r#"z"#, r#"{"#,
     r#"|"#, r#"}"#, r#"~"#,
 ];
+
+pub const PHRED64_MIN_USIZE: usize = 64;
+pub const PHRED64_MAX_USIZE: usize = 126;
+pub const PHRED64_SCORE_MIN_USIZE: usize = 0;
+pub const PHRED64_SCORE_MAX_USIZE: usize = 62;
+pub const PHRED64_MIN_U8: u8 = 64;
+pub const PHRED64_MAX_U8: u8 = 126;
+pub const PHRED64_SCORE_MIN_U8: u8 = 0;
+pub const PHRED64_SCORE_MAX_U8: u8 = 62;
+pub const PHRED64_MIN_U64: u64 = 64;
+pub const PHRED64_MAX_U64: u64 = 126;
+pub const PHRED64_SCORE_MIN_U64: u64 = 0;
+pub const PHRED64_SCORE_MAX_U64: u64 = 62;
+/// Phred64 range: 64-126
+pub const PHRED64_RANGE: RangeInclusive<usize> = PHRED64_MIN_USIZE..=PHRED64_MAX_USIZE;
+/// Phred64 range: 0-62
+pub const PHRED64_SCORE_RANGE: RangeInclusive<usize> = PHRED64_SCORE_MIN_USIZE..=PHRED64_SCORE_MAX_USIZE;
+
 lazy_static! {
     /// Phred64 charset as hashset: ASCII 64-126
-    pub static ref PHRED64_HASHSET_U8: HashSet<u8> = new_u8_hashset(&PHRED64_U8);
+    pub static ref PHRED64_HASHSET: HashSet<u8> = new_hashset(&PHRED64_ENCODE);
 }
 lazy_static! {
     /// Phred64 charset as hashset: ASCII 64-126
@@ -84,7 +176,7 @@ lazy_static! {
 
 lazy_static!{
     /// This is the quality score shifted 64 so if the u8 is 64, the score is 0. We can look that up with this hashmap.
-    pub static ref PHRED64_HASHMAP_U8: HashMap<u8, u8> = vec![
+    pub static ref PHRED64_HASHMAP_DECODE: HashMap<u8, u8> = vec![
         (b'@', 0), (b'A', 1), (b'B', 2), (b'C', 3), (b'D', 4), (b'E', 5), (b'F', 6), (b'G', 7), (b'H', 8), (b'I', 9), (b'J', 10), (b'K', 11), (b'L', 12), (b'M', 13), (b'N', 14), (b'O', 15),
         (b'P', 16), (b'Q', 17), (b'R', 18), (b'S', 19), (b'T', 20), (b'U', 21), (b'V', 22), (b'W', 23), (b'X', 24), (b'Y', 25), (b'Z', 26), (0x005B, 27), (0x005C, 28), (0x005D, 29), (b'^', 30),
         (b'_', 31), (b'`', 32), (b'a', 33), (b'b', 34), (b'c', 35), (b'd', 36), (b'e', 37), (b'f', 38), (b'g', 39), (b'h', 40), (b'i', 41), (b'j', 42), (b'k', 43), (b'l', 44), (b'm', 45), (b'n', 46),
@@ -94,7 +186,7 @@ lazy_static!{
 
 lazy_static!{
     /// This is the quality score shifted 64 so if the u8 is 64, the score is 0. We can look that up with this hashmap.
-    pub static ref PHRED64_HASHMAP_ENCODE_U8: HashMap<u8, u8> = vec![
+    pub static ref PHRED64_HASHMAP_ENCODE: HashMap<u8, u8> = vec![
         (0, b'@'), (1, b'A'), (2, b'B'), (3, b'C'), (4, b'D'), (5, b'E'), (6, b'F'), (7, b'G'), (8, b'H'), (9, b'I'), (10, b'J'), (11, b'K'), (12, b'L'), (13, b'M'), (14, b'N'), (15, b'O'),
         (16, b'P'), (17, b'Q'), (18, b'R'), (19, b'S'), (20, b'T'), (21, b'U'), (22, b'V'), (23, b'W'), (24, b'X'), (25, b'Y'), (26, b'Z'), (27, 0x005B), (28, 0x005C), (29, 0x005D), (30, b'^'),
         (31, b'_'), (32, b'`'), (33, b'a'), (34, b'b'), (35, b'c'), (36, b'd'), (37, b'e'), (38, b'f'), (39, b'g'), (40, b'h'), (41, b'i'), (42, b'j'), (43, b'k'), (44, b'l'), (45, b'm'), (46, b'n'),
@@ -103,7 +195,7 @@ lazy_static!{
 }
 
 /// Solexa/Illumina 1.0 charset: ASCII 59-126.
-pub const SOLEXA_U8: [u8; 68] = [
+pub const SOLEXA_ENCODE: [u8; 68] = [
     b';', b'<', b'=', b'>', b'?', b'@', b'A', b'B', b'C', b'D', b'E', b'F', b'G', b'H', b'I', b'J',
     b'K', b'L', b'M', b'N', b'O', b'P', b'Q', b'R', b'S', b'T', b'U', b'V', b'W', b'X', b'Y', b'Z',
     0x005B, 0x005C, 0x005D, b'^', b'_', b'`', b'a', b'b', b'c', b'd', b'e', b'f', b'g', b'h', b'i',
@@ -122,15 +214,33 @@ pub const SOLEXA_STR: [&str; 68] = [
 ];
 lazy_static! {
     /// Solexa/Illumina 1.0 charset: ASCII 59-126.
-    pub static ref SOLEXA_HASHSET_U8: HashSet<u8> = new_u8_hashset(&SOLEXA_U8);
+    pub static ref SOLEXA_HASHSET: HashSet<u8> = new_hashset(&SOLEXA_ENCODE);
 }
 lazy_static! {
     /// Solexa/Illumina 1.0 charset: ASCII 59-126.
     pub static ref SOLEXA_HASHSET_STR: HashSet<&'static str> = new_str_hashset(&SOLEXA_STR);
 }
 
+lazy_static!{
+    pub static ref SOLEXA_HASHMAP_DECODE: HashMap<u8, u8> = vec![
+        (b'@', 0), (b'A', 1), (b'B', 2), (b'C', 3), (b'D', 4), (b'E', 5), (b'F', 6), (b'G', 7), (b'H', 8), (b'I', 9), (b'J', 10), (b'K', 11), (b'L', 12), (b'M', 13), (b'N', 14), (b'O', 15),
+        (b'P', 16), (b'Q', 17), (b'R', 18), (b'S', 19), (b'T', 20), (b'U', 21), (b'V', 22), (b'W', 23), (b'X', 24), (b'Y', 25), (b'Z', 26), (0x005B, 27), (0x005C, 28), (0x005D, 29), (b'^', 30),
+        (b'_', 31), (b'`', 32), (b'a', 33), (b'b', 34), (b'c', 35), (b'd', 36), (b'e', 37), (b'f', 38), (b'g', 39), (b'h', 40), (b'i', 41), (b'j', 42), (b'k', 43), (b'l', 44), (b'm', 45), (b'n', 46),
+        (b'o', 47), (b'p', 48), (b'q', 49), (b'r', 50), (b's', 51), (b't', 52), (b'u', 53), (b'v', 54), (b'w', 55), (b'x', 56), (b'y', 57), (b'z', 58), (b'{', 59), (b'|', 60), (b'}', 61), (b'~', 62),
+    ].into_iter().collect();
+}
+
+lazy_static!{
+    pub static ref SOLEXA_HASHMAP_ENCODE: HashMap<u8, u8> = vec![
+        (b'@', 0), (b'A', 1), (b'B', 2), (b'C', 3), (b'D', 4), (b'E', 5), (b'F', 6), (b'G', 7), (b'H', 8), (b'I', 9), (b'J', 10), (b'K', 11), (b'L', 12), (b'M', 13), (b'N', 14), (b'O', 15),
+        (b'P', 16), (b'Q', 17), (b'R', 18), (b'S', 19), (b'T', 20), (b'U', 21), (b'V', 22), (b'W', 23), (b'X', 24), (b'Y', 25), (b'Z', 26), (0x005B, 27), (0x005C, 28), (0x005D, 29), (b'^', 30),
+        (b'_', 31), (b'`', 32), (b'a', 33), (b'b', 34), (b'c', 35), (b'd', 36), (b'e', 37), (b'f', 38), (b'g', 39), (b'h', 40), (b'i', 41), (b'j', 42), (b'k', 43), (b'l', 44), (b'm', 45), (b'n', 46),
+        (b'o', 47), (b'p', 48), (b'q', 49), (b'r', 50), (b's', 51), (b't', 52), (b'u', 53), (b'v', 54), (b'w', 55), (b'x', 56), (b'y', 57), (b'z', 58), (b'{', 59), (b'|', 60), (b'}', 61), (b'~', 62),
+    ].into_iter().collect();
+}
+
 /// Sanger charset: ASCII 33-126. Used by nanopore (u8-33)
-pub const SANGER_U8: [u8; 94] = [
+pub const SANGER_ENCODE: [u8; 94] = [
     b'!', b'"', b'#', b'$', b'%', b'&', 0x0027, b'(', b')', b'*', b'+', b',', b'-', b'.', b'/',
     b'0', b'1', b'2', b'3', b'4', b'5', b'6', b'7', b'8', b'9', b':', b';', b'<', b'=', b'>', b'?',
     b'@', b'A', b'B', b'C', b'D', b'E', b'F', b'G', b'H', b'I', b'J', b'K', b'L', b'M', b'N', b'O',
@@ -152,7 +262,7 @@ pub const SANGER_STR: [&str; 94] = [
 
 lazy_static!{
     /// This is the quality score shifted 33 so if the u8 is 33, the score is 0. We can look that up with this hashmap. Used by nanopore (u8-33)
-    pub static ref SANGER_HASHMAP_DECODE_U8: HashMap<u8, u8> = vec![
+    pub static ref SANGER_HASHMAP_DECODE: HashMap<u8, u8> = vec![
         (b'!', 0), (b'"', 1), (b'#', 2), (b'$', 3), (b'%', 4), (b'&', 5), (0x0027, 6), (b'(', 7), (b')', 8), (b'*', 9), (b'+', 10), (b',', 11), (b'-', 12), (b'.', 13), (b'/', 14),
         (b'0', 15), (b'1', 16), (b'2', 17), (b'3',18), (b'4', 19), (b'5', 20), (b'6', 21), (b'7', 22), (b'8', 23), (b'9', 24), (b':', 25), (b';', 26), (b'<', 27), (b'=', 28), (b'>', 29), (b'?', 30),
         (b'@', 31), (b'A', 32), (b'B', 33), (b'C', 34), (b'D', 35), (b'E', 36), (b'F', 37), (b'G', 38), (b'H', 39), (b'I', 40), (b'J', 41), (b'K', 42), 
@@ -164,7 +274,7 @@ lazy_static!{
 
 lazy_static!{
     /// This is the quality score shifted 33 so if the u8 is 33, the score is 0. We can look that up with this hashmap. Used by nanopore (u8-33)
-    pub static ref SANGER_HASHMAP_ENCODE_U8: HashMap<u8, u8> = vec![
+    pub static ref SANGER_HASHMAP_ENCODE: HashMap<u8, u8> = vec![
         (0 , b'!'), (1, b'"'), (2, b'#'), (3, b'$'), (4, b'%'), (5, b'&'), (6, 0x0027), (7, b'('), (8, b')'), (9, b'*'), (10, b'+'), (11, b','), (12, b'-'), (13, b'.'), (14, b'/'),
         (15, b'0'), (16, b'1'), (17, b'2'), (18, b'3'), (19, b'4'), (20, b'5'), (21, b'6'), (22, b'7'), (23, b'8'), (24, b'9'), (25, b':'), (26, b';'), (27, b'<'), (28, b'='), (29, b'>'), (30, b'?'),
         (31, b'@'), (32, b'A'), (33, b'B'), (34, b'C'), (35, b'D'), (36, b'E'), (37, b'F'), (38, b'G'), (39, b'H'), (40, b'I'), (41, b'J'), (42, b'K'),
@@ -174,173 +284,42 @@ lazy_static!{
     ].into_iter().collect();
 }
 
+pub const SANGER_MIN: usize = 33;
+pub const SANGER_MAX: usize = 126;
+pub const SANGER_SCORE_MIN: usize = 0;
+pub const SANGER_SCORE_MAX: usize = 93;
+/// Sanger range: 33-126
+pub const SANGER_RANGE: RangeInclusive<usize> = SANGER_MIN..=SANGER_MAX;
+/// Sanger range: 0-93
+pub const SANGER_SCORE_RANGE: RangeInclusive<usize> = SANGER_MIN..=SANGER_MAX;
+pub const SANGER_SCORE: [u8; 94] = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93];
+
 lazy_static! {
     /// Sanger charset as hashset: ASCII 33-126. Used by nanopore (u8-33)
-    pub static ref SANGER_HASHSET_U8: HashSet<u8> = new_u8_hashset(&SANGER_U8);
+    pub static ref SANGER_HASHSET: HashSet<u8> = new_hashset(&SANGER_ENCODE);
 }
 lazy_static! {
     /// Sanger charset as hashset: ASCII 33-126. Used by nanopore (u8-33)
     pub static ref SANGER_HASHSET_STR: HashSet<&'static str> = new_str_hashset(&SANGER_STR);
 }
 
-/// Flags In Order: read paired: 1, read mapped in proper pair: 2, read unmapped: 4, mate unmapped: 8, read reverse strand: 16, mate reverse strand: 32, first in pair: 64, second in pair: 128, not primary alignment: 256, read fails quality checks: 512, read is PCR or optical duplicate: 1024, supplementary alignment: 2048
-pub const FLAGS_U16: [u16; 12] = [1,2,4,8,16,32,64,128,256,512,1024,2048];
-lazy_static! {
-    /// Common alignment explanations with their associated bit
-    pub static ref FLAGS_HASHMAP_U16: HashMap<&'static str, u16> = vec![("is_paired",1), ("is_proper_pair",2), ("reads unmapped", 4), ("mate unmapped", 8), ("read reverse strand", 16), ("mate reverse strand", 32), ("first in pair", 64), ("second in pair", 128), ("not primary alignment",256), ("read fails quality checks", 512), ("read is PCR or optical duplicate", 1024), ("supplementary alignment",2048)].into_iter().collect();
-}
-
-
-
-
-
-// pub const TAGS: [u8; 1] = ["AM", "AS", ];
-
-// ub enum Tag {
-//     /// (`AM`).
-//     MinMappingQuality,
-//     /// (`AS`).
-//     AlignmentScore,
-//     /// (`BC`).
-//     SampleBarcodeSequence,
-//     /// (`BQ`).
-//     BaseAlignmentQualityOffsets,
-//     /// (`BZ`).
-//     OriginalUmiQualityScores,
-//     /// (`CB`).
-//     CellBarcodeId,
-//     /// (`CC`).
-//     NextHitReferenceSequenceName,
-//     /// (`CG`).
-//     Cigar,
-//     /// (`CM`).
-//     ColorEditDistance,
-//     /// (`CO`).
-//     Comment,
-//     /// (`CP`).
-//     NextHitPosition,
-//     /// (`CQ`).
-//     ColarQualityScores,
-//     /// (`CR`).
-//     CellBarcodeSequence,
-//     /// (`CS`).
-//     ColorSequence,
-//     /// (`CT`).
-//     CompleteReadAnnotations,
-//     /// (`CY`).
-//     CellBarcodeQualityScores,
-//     /// (`E2`).
-//     NextHitSequence,
-//     /// (`FI`).
-//     SegmentIndex,
-//     /// (`FS`).
-//     SegmentSuffix,
-//     /// (`FZ`).
-//     AlternativeSequence,
-//     /// (`GC`).
-//     ReservedGc,
-//     /// (`GQ`).
-//     ReservedGq,
-//     /// (`GS`).
-//     ReservedGs,
-//     /// (`H0`).
-//     PerfectHitCount,
-//     /// (`H1`).
-//     OneDifferenceHitCount,
-//     /// (`H2`).
-//     TwoDifferenceHitCount,
-//     /// (`HI`).
-//     HitIndex,
-//     /// (`IH`).
-//     TotalHitCount,
-//     /// (`LB`).
-//     Library,
-//     /// (`MC`).
-//     MateCigar,
-//     /// (`MD`).
-//     MismatchedPositions,
-//     /// (`MF`).
-//     ReservedMf,
-//     /// (`MI`).
-//     UmiId,
-//     /// (`MQ`).
-//     MateMappingQuality,
-//     /// (`NH`).
-//     AlignmentHitCount,
-//     /// (`NM`).
-//     EditDistance,
-//     /// (`OA`).
-//     OriginalAlignment,
-//     /// (`OC`).
-//     OriginalCigar,
-//     /// (`OP`).
-//     OriginalPosition,
-//     /// (`OQ`).
-//     OriginalQualityScores,
-//     /// (`OX`).
-//     OriginalUmiBarcodeSequence,
-//     /// (`PG`).
-//     Program,
-//     /// (`PQ`).
-//     TemplateLikelihood,
-//     /// (`PT`).
-//     PaddedReadAnnotations,
-//     /// (`PU`).
-//     PlatformUnit,
-//     /// (`Q2`).
-//     MateQualityScores,
-//     /// (`QT`).
-//     SampleBarcodeQualityScores,
-//     /// (`QX`).
-//     UmiQualityScores,
-//     /// (`R2`).
-//     MateSequence,
-//     /// (`RG`).
-//     ReadGroup,
-//     /// (`RT`).
-//     ReservedRt,
-//     /// (`RX`).
-//     UmiSequence,
-//     /// (`S2`).
-//     ReservedS2,
-//     /// (`SA`).
-//     OtherAlignments,
-//     /// (`SM`).
-//     TemplateMappingQuality,
-//     /// (`SQ`).
-//     ReservedSq,
-//     /// (`TC`).
-//     SegmentCount,
-//     /// (`TS`).
-//     TranscriptStrand,
-//     /// (`U2`).
-//     NextHitQualityScores,
-//     /// (`UQ`).
-//     SegmentLikelihood,
-//     /// Any other non-standard tag.
-//     Other(String),
-// }
-
-// If any other tags please create your own constant and make a pull request!
-
-
 // #[cfg(test)]
 // mod tests {
-//     use super::{PHRED33_U8, PHRED64_U8, SOLEXA_U8};
+//     use super::{PHRED33, PHRED64, SOLEXA};
 //     #[test]
 //     fn test_phred33() {
 //         let dec: Vec<u8> = (33..127).collect();
-//         assert_eq!(dec, PHRED33_U8);
+//         assert_eq!(dec, PHRED33);
 //     }
 //     #[test]
 //     fn test_phred64() {
 //         let dec: Vec<u8> = (64..127).collect();
-//         assert_eq!(dec, PHRED64_U8);
+//         assert_eq!(dec, PHRED64);
 //     }
 //     #[test]
 //     fn test_solexa() {
 //         let dec: Vec<u8> = (59..127).collect();
-//         assert_eq!(dec, SOLEXA_U8);
+//         assert_eq!(dec, SOLEXA);
 //     }
 // }
 
